@@ -5,14 +5,14 @@
 
     Core locale representation and locale data access.
 
-    :copyright: (c) 2013 by the Babel Team.
+    :copyright: (c) 2013-2022 by the Babel Team.
     :license: BSD, see LICENSE for more details.
 """
 
+import pickle
 import os
 
 from babel import localedata
-from babel._compat import pickle, string_types
 from babel.plural import PluralRule
 
 __all__ = ['UnknownLocaleError', 'Locale', 'default_locale', 'negotiate_locale',
@@ -39,7 +39,7 @@ def get_global(key):
     information independent of individual locales.
 
     >>> get_global('zone_aliases')['UTC']
-    u'Etc/GMT'
+    u'Etc/UTC'
     >>> get_global('zone_territories')['Europe/Berlin']
     u'DE'
 
@@ -262,7 +262,7 @@ class Locale(object):
             return None
         elif isinstance(identifier, Locale):
             return identifier
-        elif not isinstance(identifier, string_types):
+        elif not isinstance(identifier, str):
             raise TypeError('Unexpected value for identifier: %r' % (identifier,))
 
         parts = parse_locale(identifier, sep=sep)
@@ -379,7 +379,7 @@ class Locale(object):
             locale = self
         locale = Locale.parse(locale)
         retval = locale.languages.get(self.language)
-        if self.territory or self.script or self.variant:
+        if retval and (self.territory or self.script or self.variant):
             details = []
             if self.script:
                 details.append(locale.scripts.get(self.script))
@@ -857,11 +857,11 @@ class Locale(object):
         .. note:: The format of the value returned may change between
                   Babel versions.
 
-        >>> Locale('en').list_patterns['start']
+        >>> Locale('en').list_patterns['standard']['start']
         u'{0}, {1}'
-        >>> Locale('en').list_patterns['end']
+        >>> Locale('en').list_patterns['standard']['end']
         u'{0}, and {1}'
-        >>> Locale('en_GB').list_patterns['end']
+        >>> Locale('en_GB').list_patterns['standard']['end']
         u'{0} and {1}'
         """
         return self._data['list_patterns']
@@ -1048,6 +1048,12 @@ def parse_locale(identifier, sep='_'):
     ('zh', 'CN', None, None)
     >>> parse_locale('zh_Hans_CN')
     ('zh', 'CN', 'Hans', None)
+    >>> parse_locale('ca_es_valencia')
+    ('ca', 'ES', None, 'VALENCIA')
+    >>> parse_locale('en_150')
+    ('en', '150', None, None)
+    >>> parse_locale('en_us_posix')
+    ('en', 'US', None, 'POSIX')
 
     The default component separator is "_", but a different separator can be
     specified using the `sep` parameter:
@@ -1107,7 +1113,7 @@ def parse_locale(identifier, sep='_'):
     if parts:
         if len(parts[0]) == 4 and parts[0][0].isdigit() or \
                 len(parts[0]) >= 5 and parts[0][0].isalpha():
-            variant = parts.pop()
+            variant = parts.pop().upper()
 
     if parts:
         raise ValueError('%r is not a valid locale identifier' % identifier)
@@ -1118,7 +1124,7 @@ def parse_locale(identifier, sep='_'):
 def get_locale_identifier(tup, sep='_'):
     """The reverse of :func:`parse_locale`.  It creates a locale identifier out
     of a ``(language, territory, script, variant)`` tuple.  Items can be set to
-    ``None`` and trailing ``None``\s can also be left out of the tuple.
+    ``None`` and trailing ``None``\\s can also be left out of the tuple.
 
     >>> get_locale_identifier(('de', 'DE', None, '1999'))
     'de_DE_1999'
