@@ -12,7 +12,7 @@
      * ``LC_ALL``, and
      * ``LANG``
 
-    :copyright: (c) 2013 by the Babel Team.
+    :copyright: (c) 2013-2022 by the Babel Team.
     :license: BSD, see LICENSE for more details.
 """
 
@@ -27,7 +27,15 @@ from bisect import bisect_right
 
 from babel.core import default_locale, get_global, Locale
 from babel.util import UTC, LOCALTZ
-from babel._compat import string_types, integer_types, number_types
+
+# "If a given short metazone form is known NOT to be understood in a given
+#  locale and the parent locale has this value such that it would normally
+#  be inherited, the inheritance of this value can be explicitly disabled by
+#  use of the 'no inheritance marker' as the value, which is 3 simultaneous [sic]
+#  empty set characters ( U+2205 )."
+#  - https://www.unicode.org/reports/tr35/tr35-dates.html#Metazone_Names
+
+NO_INHERITANCE_MARKER = u'\u2205\u2205\u2205'
 
 
 LC_TIME = default_locale('LC_TIME')
@@ -49,10 +57,10 @@ def _get_dt_and_tzinfo(dt_or_tzinfo):
     if dt_or_tzinfo is None:
         dt = datetime.now()
         tzinfo = LOCALTZ
-    elif isinstance(dt_or_tzinfo, string_types):
+    elif isinstance(dt_or_tzinfo, str):
         dt = None
         tzinfo = get_timezone(dt_or_tzinfo)
-    elif isinstance(dt_or_tzinfo, integer_types):
+    elif isinstance(dt_or_tzinfo, int):
         dt = None
         tzinfo = UTC
     elif isinstance(dt_or_tzinfo, (datetime, time)):
@@ -67,6 +75,21 @@ def _get_dt_and_tzinfo(dt_or_tzinfo):
     return dt, tzinfo
 
 
+def _get_tz_name(dt_or_tzinfo):
+    """
+    Get the timezone name out of a time, datetime, or tzinfo object.
+
+    :rtype: str
+    """
+    dt, tzinfo = _get_dt_and_tzinfo(dt_or_tzinfo)
+    if hasattr(tzinfo, 'zone'):  # pytz object
+        return tzinfo.zone
+    elif hasattr(tzinfo, 'key') and tzinfo.key is not None:  # ZoneInfo object
+        return tzinfo.key
+    else:
+        return tzinfo.tzname(dt or datetime.utcnow())
+
+
 def _get_datetime(instant):
     """
     Get a datetime out of an "instant" (date, time, datetime, number).
@@ -99,7 +122,7 @@ def _get_datetime(instant):
     """
     if instant is None:
         return datetime_.utcnow()
-    elif isinstance(instant, integer_types) or isinstance(instant, float):
+    elif isinstance(instant, int) or isinstance(instant, float):
         return datetime_.utcfromtimestamp(instant)
     elif isinstance(instant, time):
         return datetime_.combine(date.today(), instant)
@@ -149,7 +172,7 @@ def _get_time(time, tzinfo=None):
     """
     if time is None:
         time = datetime.utcnow()
-    elif isinstance(time, number_types):
+    elif isinstance(time, (int, float)):
         time = datetime.utcfromtimestamp(time)
     if time.tzinfo is None:
         time = time.replace(tzinfo=UTC)
@@ -177,7 +200,7 @@ def get_timezone(zone=None):
     """
     if zone is None:
         return LOCALTZ
-    if not isinstance(zone, string_types):
+    if not isinstance(zone, str):
         return zone
     try:
         return _pytz.timezone(zone)
@@ -198,11 +221,21 @@ def get_next_timezone_transition(zone=None, dt=None):
     Transition information can only be provided for timezones returned by
     the :func:`get_timezone` function.
 
+    This function is pending deprecation with no replacement planned in the
+    Babel library.
+
     :param zone: the timezone for which the transition should be looked up.
                  If not provided the local timezone is used.
     :param dt: the date after which the next transition should be found.
                If not given the current time is assumed.
     """
+    warnings.warn(
+        "get_next_timezone_transition() is deprecated and will be "
+        "removed in the next version of Babel. "
+        "Please see https://github.com/python-babel/babel/issues/716 "
+        "for discussion.",
+        category=DeprecationWarning,
+    )
     zone = get_timezone(zone)
     dt = _get_datetime(dt).replace(tzinfo=None)
 
@@ -232,17 +265,32 @@ def get_next_timezone_transition(zone=None, dt=None):
 class TimezoneTransition(object):
     """A helper object that represents the return value from
     :func:`get_next_timezone_transition`.
+
+    This class is pending deprecation with no replacement planned in the
+    Babel library.
+
+    :field activates:
+        The time of the activation of the timezone transition in UTC.
+    :field from_tzinfo:
+        The timezone from where the transition starts.
+    :field to_tzinfo:
+        The timezone for after the transition.
+    :field reference_date:
+        The reference date that was provided.  This is the `dt` parameter
+        to the :func:`get_next_timezone_transition`.
     """
 
     def __init__(self, activates, from_tzinfo, to_tzinfo, reference_date=None):
-        #: the time of the activation of the timezone transition in UTC.
+        warnings.warn(
+            "TimezoneTransition is deprecated and will be "
+            "removed in the next version of Babel. "
+            "Please see https://github.com/python-babel/babel/issues/716 "
+            "for discussion.",
+            category=DeprecationWarning,
+        )
         self.activates = activates
-        #: the timezone from where the transition starts.
         self.from_tzinfo = from_tzinfo
-        #: the timezone for after the transition.
         self.to_tzinfo = to_tzinfo
-        #: the reference date that was provided.  This is the `dt` parameter
-        #: to the :func:`get_next_timezone_transition`.
         self.reference_date = reference_date
 
     @property
@@ -294,7 +342,7 @@ def get_day_names(width='wide', context='format', locale=LC_TIME):
     >>> get_day_names('short', locale='en_US')[1]
     u'Tu'
     >>> get_day_names('abbreviated', locale='es')[1]
-    u'mar.'
+    u'mar'
     >>> get_day_names('narrow', context='stand-alone', locale='de_DE')[1]
     u'D'
 
@@ -311,7 +359,7 @@ def get_month_names(width='wide', context='format', locale=LC_TIME):
     >>> get_month_names('wide', locale='en_US')[1]
     u'January'
     >>> get_month_names('abbreviated', locale='es')[1]
-    u'ene.'
+    u'ene'
     >>> get_month_names('narrow', context='stand-alone', locale='de_DE')[1]
     u'J'
 
@@ -491,13 +539,9 @@ def get_timezone_location(dt_or_tzinfo=None, locale=LC_TIME, return_city=False):
     :return: the localized timezone name using location format
 
     """
-    dt, tzinfo = _get_dt_and_tzinfo(dt_or_tzinfo)
     locale = Locale.parse(locale)
 
-    if hasattr(tzinfo, 'zone'):
-        zone = tzinfo.zone
-    else:
-        zone = tzinfo.tzname(dt or datetime.utcnow())
+    zone = _get_tz_name(dt_or_tzinfo)
 
     # Get the canonical time-zone code
     zone = get_global('zone_aliases').get(zone, zone)
@@ -512,7 +556,7 @@ def get_timezone_location(dt_or_tzinfo=None, locale=LC_TIME, return_city=False):
         territory = 'ZZ'  # invalid/unknown
     territory_name = locale.territories[territory]
     if not return_city and territory and len(get_global('territory_zones').get(territory, [])) == 1:
-        return region_format % (territory_name)
+        return region_format % territory_name
 
     # Otherwise, include the city in the output
     fallback_format = locale.zone_formats['fallback']
@@ -583,7 +627,7 @@ def get_timezone_name(dt_or_tzinfo=None, width='long', uncommon=False,
     format.
 
     For more information see `LDML Appendix J: Time Zone Display Names
-    <http://www.unicode.org/reports/tr35/#Time_Zone_Fallback>`_
+    <https://www.unicode.org/reports/tr35/#Time_Zone_Fallback>`_
 
     .. versionadded:: 0.9
 
@@ -610,10 +654,7 @@ def get_timezone_name(dt_or_tzinfo=None, width='long', uncommon=False,
     dt, tzinfo = _get_dt_and_tzinfo(dt_or_tzinfo)
     locale = Locale.parse(locale)
 
-    if hasattr(tzinfo, 'zone'):
-        zone = tzinfo.zone
-    else:
-        zone = tzinfo.tzname(dt)
+    zone = _get_tz_name(dt_or_tzinfo)
 
     if zone_variant is None:
         if dt is None:
@@ -642,8 +683,13 @@ def get_timezone_name(dt_or_tzinfo=None, width='long', uncommon=False,
     if metazone:
         metazone_info = locale.meta_zones.get(metazone, {})
         if width in metazone_info:
-            if zone_variant in metazone_info[width]:
-                return metazone_info[width][zone_variant]
+            name = metazone_info[width].get(zone_variant)
+            if width == 'short' and name == NO_INHERITANCE_MARKER:
+                # If the short form is marked no-inheritance,
+                # try to fall back to the long name instead.
+                name = metazone_info.get('long', {}).get(zone_variant)
+            if name:
+                return name
 
     # If we have a concrete datetime, we assume that the result can't be
     # independent of daylight savings time, so we return the GMT offset
@@ -927,7 +973,7 @@ def format_timedelta(delta, granularity='second', threshold=.85,
             pattern = None
             for patterns in _iter_patterns(unit):
                 if patterns is not None:
-                    pattern = patterns[plural_form]
+                    pattern = patterns.get(plural_form) or patterns.get('other')
                     break
             # This really should not happen
             if pattern is None:
@@ -968,7 +1014,7 @@ def format_interval(start, end, skeleton=None, tzinfo=None, fuzzy=True, locale=L
     u'15.\u201317.1.2016'
 
     >>> format_interval(time(12, 12), time(16, 16), "Hm", locale="en_GB")
-    '12:12 \u2013 16:16'
+    '12:12\u201316:16'
 
     >>> format_interval(time(5, 12), time(16, 16), "hm", locale="en_US")
     '5:12 AM \u2013 4:16 PM'
@@ -1004,7 +1050,7 @@ def format_interval(start, end, skeleton=None, tzinfo=None, fuzzy=True, locale=L
     locale = Locale.parse(locale)
 
     # NB: The quote comments below are from the algorithm description in
-    #     http://www.unicode.org/reports/tr35/tr35-dates.html#intervalFormats
+    #     https://www.unicode.org/reports/tr35/tr35-dates.html#intervalFormats
 
     # > Look for the intervalFormatItem element that matches the "skeleton",
     # > starting in the current locale and then following the locale fallback
@@ -1080,12 +1126,16 @@ def get_period_id(time, tzinfo=None, type=None, locale=LC_TIME):
     locale = Locale.parse(locale)
 
     # The LDML rules state that the rules may not overlap, so iterating in arbitrary
-    # order should be alright.
-    for rule_id, rules in locale.day_period_rules.get(type, {}).items():
+    # order should be alright, though `at` periods should be preferred.
+    rulesets = locale.day_period_rules.get(type, {}).items()
+
+    for rule_id, rules in rulesets:
         for rule in rules:
             if "at" in rule and rule["at"] == seconds_past_midnight:
                 return rule_id
 
+    for rule_id, rules in rulesets:
+        for rule in rules:
             start_ok = end_ok = False
 
             if "from" in rule and seconds_past_midnight >= rule["from"]:
@@ -1096,8 +1146,8 @@ def get_period_id(time, tzinfo=None, type=None, locale=LC_TIME):
                 end_ok = True
             if "before" in rule and seconds_past_midnight < rule["before"]:
                 end_ok = True
-            if "after" in rule and seconds_past_midnight > rule["after"]:
-                start_ok = True
+            if "after" in rule:
+                raise NotImplementedError("'after' is deprecated as of CLDR 29.")
 
             if start_ok and end_ok:
                 return rule_id
@@ -1108,7 +1158,11 @@ def get_period_id(time, tzinfo=None, type=None, locale=LC_TIME):
         return "pm"
 
 
-def parse_date(string, locale=LC_TIME):
+class ParseError(ValueError):
+    pass
+
+
+def parse_date(string, locale=LC_TIME, format='medium'):
     """Parse a date from a string.
 
     This function uses the date format for the locale as a hint to determine
@@ -1121,14 +1175,19 @@ def parse_date(string, locale=LC_TIME):
 
     :param string: the string containing the date
     :param locale: a `Locale` object or a locale identifier
+    :param format: the format to use (see ``get_date_format``)
     """
+    numbers = re.findall(r'(\d+)', string)
+    if not numbers:
+        raise ParseError("No numbers were found in input")
+
     # TODO: try ISO format first?
-    format = get_date_format(locale=locale).pattern.lower()
-    year_idx = format.index('y')
-    month_idx = format.index('m')
+    format_str = get_date_format(format=format, locale=locale).pattern.lower()
+    year_idx = format_str.index('y')
+    month_idx = format_str.index('m')
     if month_idx < 0:
-        month_idx = format.index('l')
-    day_idx = format.index('d')
+        month_idx = format_str.index('l')
+    day_idx = format_str.index('d')
 
     indexes = [(year_idx, 'Y'), (month_idx, 'M'), (day_idx, 'D')]
     indexes.sort()
@@ -1137,7 +1196,6 @@ def parse_date(string, locale=LC_TIME):
     # FIXME: this currently only supports numbers, but should also support month
     #        names, both in the requested locale, and english
 
-    numbers = re.findall('(\d+)', string)
     year = numbers[indexes['Y']]
     if len(year) == 2:
         year = 2000 + int(year)
@@ -1150,7 +1208,7 @@ def parse_date(string, locale=LC_TIME):
     return date(year, month, day)
 
 
-def parse_time(string, locale=LC_TIME):
+def parse_time(string, locale=LC_TIME, format='medium'):
     """Parse a time from a string.
 
     This function uses the time format for the locale as a hint to determine
@@ -1161,29 +1219,42 @@ def parse_time(string, locale=LC_TIME):
 
     :param string: the string containing the time
     :param locale: a `Locale` object or a locale identifier
+    :param format: the format to use (see ``get_time_format``)
     :return: the parsed time
     :rtype: `time`
     """
+    numbers = re.findall(r'(\d+)', string)
+    if not numbers:
+        raise ParseError("No numbers were found in input")
+
     # TODO: try ISO format first?
-    format = get_time_format(locale=locale).pattern.lower()
-    hour_idx = format.index('h')
+    format_str = get_time_format(format=format, locale=locale).pattern.lower()
+    hour_idx = format_str.index('h')
     if hour_idx < 0:
-        hour_idx = format.index('k')
-    min_idx = format.index('m')
-    sec_idx = format.index('s')
+        hour_idx = format_str.index('k')
+    min_idx = format_str.index('m')
+    sec_idx = format_str.index('s')
 
     indexes = [(hour_idx, 'H'), (min_idx, 'M'), (sec_idx, 'S')]
     indexes.sort()
     indexes = dict([(item[1], idx) for idx, item in enumerate(indexes)])
 
-    # FIXME: support 12 hour clock, and 0-based hour specification
-    #        and seconds should be optional, maybe minutes too
-    #        oh, and time-zones, of course
+    # TODO: support time zones
 
-    numbers = re.findall('(\d+)', string)
-    hour = int(numbers[indexes['H']])
-    minute = int(numbers[indexes['M']])
-    second = int(numbers[indexes['S']])
+    # Check if the format specifies a period to be used;
+    # if it does, look for 'pm' to figure out an offset.
+    hour_offset = 0
+    if 'a' in format_str:
+        if 'pm' in string.lower():
+            hour_offset = 12
+
+    # Parse up to three numbers from the string.
+    minute = second = 0
+    hour = int(numbers[indexes['H']]) + hour_offset
+    if len(numbers) > 1:
+        minute = int(numbers[indexes['M']])
+        if len(numbers) > 2:
+            second = int(numbers[indexes['S']])
     return time(hour, minute, second)
 
 
@@ -1199,6 +1270,10 @@ class DateTimePattern(object):
     def __unicode__(self):
         return self.pattern
 
+    def __str__(self):
+        pat = self.pattern
+        return pat
+
     def __mod__(self, other):
         if type(other) is not DateTimeFormat:
             return NotImplemented
@@ -1279,7 +1354,7 @@ class DateTimeFormat(object):
         elif char == 'H':
             return self.value.hour
         elif char == 'h':
-            return (self.value.hour % 12 or 12)
+            return self.value.hour % 12 or 12
         elif char == 'm':
             return self.value.minute
         elif char == 'a':
@@ -1295,9 +1370,7 @@ class DateTimeFormat(object):
     def format_year(self, char, num):
         value = self.value.year
         if char.isupper():
-            week = self.get_week_number(self.get_day_of_year())
-            if week == 0:
-                value -= 1
+            value = self.value.isocalendar()[0]
         year = self.format(value, num)
         if num == 2:
             year = year[-2:]
@@ -1306,14 +1379,14 @@ class DateTimeFormat(object):
     def format_quarter(self, char, num):
         quarter = (self.value.month - 1) // 3 + 1
         if num <= 2:
-            return ('%%0%dd' % num) % quarter
+            return '%0*d' % (num, quarter)
         width = {3: 'abbreviated', 4: 'wide', 5: 'narrow'}[num]
         context = {'Q': 'format', 'q': 'stand-alone'}[char]
         return get_quarter_names(width, context, self.locale)[quarter]
 
     def format_month(self, char, num):
         if num <= 2:
-            return ('%%0%dd' % num) % self.value.month
+            return '%0*d' % (num, self.value.month)
         width = {3: 'abbreviated', 4: 'wide', 5: 'narrow'}[num]
         context = {'M': 'format', 'L': 'stand-alone'}[char]
         return get_month_names(width, context, self.locale)[self.value.month]
@@ -1446,7 +1519,7 @@ class DateTimeFormat(object):
                 return get_timezone_gmt(self.value, width='iso8601', locale=self.locale)
 
     def format(self, value, length):
-        return ('%%0%dd' % length) % value
+        return '%0*d' % (length, value)
 
     def get_day_of_year(self, date=None):
         if date is None:
@@ -1481,8 +1554,20 @@ class DateTimeFormat(object):
         if first_day < 0:
             first_day += 7
         week_number = (day_of_period + first_day - 1) // 7
+
         if 7 - first_day >= self.locale.min_week_days:
             week_number += 1
+
+        if self.locale.first_week_day == 0:
+            # Correct the weeknumber in case of iso-calendar usage (first_week_day=0).
+            # If the weeknumber exceeds the maximum number of weeks for the given year
+            # we must count from zero.For example the above calculation gives week 53
+            # for 2018-12-31. By iso-calender definition 2018 has a max of 52
+            # weeks, thus the weeknumber must be 53-52=1.
+            max_weeks = date(year=self.value.year, day=28, month=12).isocalendar()[1]
+            if week_number > max_weeks:
+                week_number -= max_weeks
+
         return week_number
 
 
@@ -1503,7 +1588,7 @@ PATTERN_CHARS = {
 }
 
 #: The pattern characters declared in the Date Field Symbol Table
-#: (http://www.unicode.org/reports/tr35/tr35-dates.html#Date_Field_Symbol_Table)
+#: (https://www.unicode.org/reports/tr35/tr35-dates.html#Date_Field_Symbol_Table)
 #: in order of decreasing magnitude.
 PATTERN_CHAR_ORDER = "GyYuUQqMLlwWdDFgEecabBChHKkjJmsSAzZOvVXx"
 
@@ -1651,7 +1736,7 @@ def split_interval_pattern(pattern):
     Split an interval-describing datetime pattern into multiple pieces.
 
     > The pattern is then designed to be broken up into two pieces by determining the first repeating field.
-    - http://www.unicode.org/reports/tr35/tr35-dates.html#intervalFormats
+    - https://www.unicode.org/reports/tr35/tr35-dates.html#intervalFormats
 
     >>> split_interval_pattern(u'E d.M. \u2013 E d.M.')
     [u'E d.M. \u2013 ', 'E d.M.']
