@@ -6,7 +6,7 @@
     Reading and writing of files in the ``gettext`` PO (portable object)
     format.
 
-    :copyright: (c) 2013-2020 by the Babel Team.
+    :copyright: (c) 2013-2022 by the Babel Team.
     :license: BSD, see LICENSE for more details.
 """
 
@@ -15,8 +15,7 @@ import os
 import re
 
 from babel.messages.catalog import Catalog, Message
-from babel.util import wraptext
-from babel._compat import text_type, cmp
+from babel.util import wraptext, _cmp
 
 
 def unescape(string):
@@ -107,7 +106,7 @@ class _NormalizedString(object):
         if not other:
             return 1
 
-        return cmp(text_type(self), text_type(other))
+        return _cmp(str(self), str(other))
 
     def __gt__(self, other):
         return self.__cmp__(other) > 0
@@ -178,7 +177,7 @@ class PoFileParser(object):
             string = ['' for _ in range(self.catalog.num_plurals)]
             for idx, translation in self.translations:
                 if idx >= self.catalog.num_plurals:
-                    self._invalid_pofile("", self.offset, "msg has more translations than num_plurals of catalog")
+                    self._invalid_pofile(u"", self.offset, "msg has more translations than num_plurals of catalog")
                     continue
                 string[idx] = translation.denormalize()
             string = tuple(string)
@@ -297,7 +296,7 @@ class PoFileParser(object):
 
         for lineno, line in enumerate(fileobj):
             line = line.strip()
-            if not isinstance(line, text_type):
+            if not isinstance(line, str):
                 line = line.decode(self.catalog.charset)
             if not line:
                 continue
@@ -319,10 +318,14 @@ class PoFileParser(object):
             self._add_message()
 
     def _invalid_pofile(self, line, lineno, msg):
+        assert isinstance(line, str)
         if self.abort_invalid:
             raise PoFileError(msg, self.catalog, line, lineno)
         print("WARNING:", msg)
-        print(u"WARNING: Problem on line {0}: {1}".format(lineno + 1, line))
+        # `line` is guaranteed to be unicode so u"{}"-interpolating would always
+        # succeed, but on Python < 2 if not in a TTY, `sys.stdout.encoding`
+        # is `None`, unicode may not be printable so we `repr()` to ASCII.
+        print(u"WARNING: Problem on line {0}: {1}".format(lineno + 1, repr(line)))
 
 
 def read_po(fileobj, locale=None, domain=None, ignore_obsolete=False, charset=None, abort_invalid=False):
@@ -330,7 +333,7 @@ def read_po(fileobj, locale=None, domain=None, ignore_obsolete=False, charset=No
     file-like object and return a `Catalog`.
 
     >>> from datetime import datetime
-    >>> from babel._compat import StringIO
+    >>> from io import StringIO
     >>> buf = StringIO('''
     ... #: main.py:1
     ... #, fuzzy, python-format
@@ -476,7 +479,7 @@ def write_po(fileobj, catalog, width=76, no_location=False, omit_header=False,
     <Message...>
     >>> catalog.add((u'bar', u'baz'), locations=[('main.py', 3)])
     <Message...>
-    >>> from babel._compat import BytesIO
+    >>> from io import BytesIO
     >>> buf = BytesIO()
     >>> write_po(buf, catalog, omit_header=True)
     >>> print(buf.getvalue().decode("utf8"))
@@ -514,7 +517,7 @@ def write_po(fileobj, catalog, width=76, no_location=False, omit_header=False,
         return normalize(key, prefix=prefix, width=width)
 
     def _write(text):
-        if isinstance(text, text_type):
+        if isinstance(text, str):
             text = text.encode(catalog.charset, 'backslashreplace')
         fileobj.write(text)
 
@@ -594,9 +597,11 @@ def write_po(fileobj, catalog, width=76, no_location=False, omit_header=False,
 
             for filename, lineno in locations:
                 if lineno and include_lineno:
-                    locs.append(u'%s:%d' % (filename.replace(os.sep, '/'), lineno))
+                    location = u'%s:%d' % (filename.replace(os.sep, '/'), lineno)
                 else:
-                    locs.append(u'%s' % filename.replace(os.sep, '/'))
+                    location = u'%s' % filename.replace(os.sep, '/')
+                if location not in locs:
+                    locs.append(location)
             _write_comment(' '.join(locs), prefix=':')
         if message.flags:
             _write('#%s\n' % ', '.join([''] + sorted(message.flags)))
